function [NEdata NEdataRefFilPack]= ...
                      NE_Viewer(filepath, ref, fs, LabelsElectrodes,band,T) 
  
% NE_Viewer(filepath) reads Enobio/StarStim (3G, released 2012) data in 
% ASCII (.txt or ".easy") or EDF format, and plots parts of it together 
% with its PSD and spectrogram.
%
% This function is provided as is as an example on how to work with NE instrument 
% data.
%
% INPUTS: 
% Enobio/StarStim (8 or 20 or 32 Ch) data file path (mandatory) and other optional
% entries: "ref" the optional reference channel (use 0 for none, 999 for 
% an all electrode average), "fs" the sampling rate in S/s, 
% "LabelsElectrodes" a structure containing channel names, and "band" a 
% 2-element array with the low and high filtering frequency, e.g., 
% band=[1 40], in Hz. Finally, "T" is an arracy specifiying initital and 
% final time to analyze (in seconds), e.g., T=[10 130].
%
% OUTPUTS: 
% Returns the original, raw data pack, NEdata, as a Matlab variable.
% preserving the units (e.g., nV, ms). It also returnes NEdataRefFilPack, 
% the processed (referenced, detreneded, filtered) data (in uV). The time
% stamp (in seconds from start of the take) is provided in the last column.
%
% SIDE EFFECTS: creation of "figures" directory if it does not exist, 
%               creation of data plot figures of all Enobio channels, etc. 
% 
%       out = NE_Viewer(filepath) plots the data in the filepath
%       (filepath includes path relative to the Matlab current working 
%       directory of full absolute path), its PSD and spectrogram for each channel, 
%       etc., for 8 channels. As default it assumes a 500 Samples/s
%       sampling rate, and does not reference the data to any other
%       channel.  The code puts figures in a directory under the working
%       directory.
%      
%       NE_Viewer(filepath, ref) references the data to a chosen channel 
%       (ref should be a Channel number from 1 to 8 (or 20 for Enobio20), 
%       or zero to force no reference - the default). A reference = 999
%       means the average of all electrodes is used as reference.
% 
%       In addition, using the call NE_View(filepath, ref, fs) allows 
%       for changing the sampling rate fs in Samples/s. 
%      
%       Use a structure with LabelsElectrodes such as 
%         >> LabelsElectrodes={'Ch1','Ch2','Ch3','Ch4','Ch5','Ch6','Ch7','Ch8'}
%       with 
%         >> NE_View(filepath, ref, fs, LabelsElectrodes) 
%       to add LabelsElectrodes to channel IDs in plots.
% 
%
% EXAMPLE: assuming the data is in a folder called "data", 
%          NE_Viewer('./data/EC.txt')   % uses all defaults
%          NE_Viewer('./data/EC.txt',1)    % spedifies reference channel
%          NE_Viewer('./data/EC.txt',1,500) % specifies Sampling rate
%
%          To specify channel LabelsElectrodes
%          NE_Viewer('./data/EC.txt',1,500, ...
%                    {'Fp1','Fp2','P3','P4','O1','O1','T7','Pz'} ) 
%
%          To specifiy a filter band other than the default:
%          NE_Viewer('data/StarStim/20120728085042_TBtACS10Hz1mA.txt',0, ...
%                                                         [],[],[1 60]);
% 
%    As another example, 
%           
%       >> [NEdata NEdataRefFilPack]=NE_Viewer('data/Enobio8.txt');
%   and then plot data vs. time 
%       >> figure; plot(NEdataRefFilPack(:,9), NEdataRefFilPack(:,1));
%
% ABOUT THE NE ASCII DATA FORMAT (July 2012): 
%       NE ASCII files contain one line per time sample. Each line contains
%       first the EEG data (8 or 20 channels, depending on the device, with
%       units in nV), followed by  three acceleration channels (aX,aY,aZ 
%       in mm/s^2- millimeters per second squared), an *optional* external 
%       input channel, a trigger flag (int32) and, finally, 
%       a timestamp in Unix time (ms from Jan 1 1970):
%
% Ch1(nV) ... Ch8or20(nV) aX(mg) aY(mg) aZ(mg) AddSensor Flags(uint32) TimeStamp (ms)
%   
% Therefore Enobio8/StarStim will have a minimum of 8+2 (10) columns, or 8+3=11 if no 
%   accelerometer or >= 8+4 (with accelerometer). In summary:
%
%   Enobio8/20
%        10 Columns: no AddSensor, no accelerometer data:
%        11 Comumns: no accelerometer, but there is AddSensor
%        13 Columns: there is accelerometer, but no Addsensor
%        14 Comumns: there is accelerometer, AddSensor.
%
%   Enobio 20 will have >= 22 and >=24 columns if acc. data present:
%        22 Columns: no AddSensor, no accelerometer
%        23 Comumns: no accelerometer, but there is AddSensor
%        25 Columns: there is accelerometer, no AddSensor 
%        26 Comumns: there is accelerometer, AddSensor.
%
%   Enobio 32 will have >=34 adn >=36 if acc data is present
%        34 Columns: no AddSensor, no accelerometer
%        35 Comumns: no accelerometer, but there is AddSensor
%        37 Columns: there is accelerometer, no AddSensor 
%        38 Comumns: there is accelerometer, AddSensor.
%
%   Author(s): G.R. - Neuroelectrics Barcelona SL 
%   Copyright 2011-13 Neuroelectrics Barcelona SL
%   $Revision: 0.1 $  $Date: 2011/11/26 $ 
%   $Revision: 0.2 $  $Date: 2012/07/16 $ 
%   $Revision: 0.3 $  $Date: 2012/11/15 $ 
%   $Revision: 0.4 $  $Date: 2013/01/11 $ 
%   $Revision: 1.0 $  $Date: 2013/01/15 $ 
%   $Revision: 1.1 $  $Date: 2013/01/15 $ 
%   $Revision: 1.2 $  $Date: 2014/08/06 $ 

%   References:
%     [1] Enobio users guide. http://neuroelectrics.com
 
%%  First we close all figures, check for optional inputs

close all; % close all windows to start with
% Now we create directory for figure output ... will warn if it exists:
% First we extract path string and filename:
[pathstr, filename, ext]=fileparts(filepath);
ProcessLogFile=[filename '-ProcessLog.txt'];
delete(ProcessLogFile);
diary(ProcessLogFile)
disp(['Processing started ' datestr(now)]);


% Try to Grab current call for the log file.
try
history = com.mathworks.mlservices.MLCommandHistoryServices.getSessionHistory;                          
historyText = char(history);    
thiscall=historyText(end,:);
disp(['This Matlab call: ' thiscall]);
catch
    disp('[Could not grab call/history. Continuing ... ]');
    
end


%% Step 0: CHECK INPUTS AND FALL BACK TO DEFAULTS AS NEEDED
% check if user has input ref, otherwise use ref=0 (no reference)
%
if ~exist('ref','var')
    ref=0; % this means data will not be referenced to a channel
    disp('Will not reference data to a specific channel');
end
% check if user has input sampling frequency, otherwise use 500 S/s
if ~exist('fs','var') || isempty(fs)  
    fs=500; %Sample/s, this is the default sampling frequency in Enobio/StarStim
      disp('Using default sampling rate of 500 S/s');
end


% check if band is specified, or use default
if ~exist('band','var') || isempty(band) 
% define frequency bandpass if not provided by user
 flow=1; %Hz - low cutoff frequency for filtering 
 fhigh=40; %Hz - high cutoff frequency for filtering
else % or use the user-provided one:
     flow=band(1);
     fhigh=band(2);
end
   disp(['Filtering set to:[' num2str(flow) ', ' num2str(fhigh) '] Hz' ] );
   bandtitle=['[' num2str(flow) ', ' num2str(fhigh) ']'];

    
disp(['Sampling rate set to ' num2str(fs) ' Samples/s']) ;


% create figures directory, set up ProcessLog diary file
figdir=[pathstr 'figures-' filename ...
                 '[' num2str(flow) ',' num2str(fhigh) ']Hz' ] ;
            
mkdir(figdir);


 
 
%% STEP 1: READ DATA AND CREATE VARIABLES, PLOT RAW DATA
% Read data, create variabless (data input has many rows and 6 columns 
% following the ASCII standard Enobio data format, with each row: 
%       Ch1(nV) ... Ch8(nV)  flags  Time(msUNIXtime)

% Read the NE ASCII or EDF data:
% We expect  TXT data to come with each time sample added as a row.
% E.g., NEdata  is    92578x9    matlab double
 

disp('Loading data ...');
if strcmp(ext,'.easy') || strcmp(ext,'.txt')|| strcmp(ext,'.TXT')
  NEdata=load(filepath); % 
  disp('Data in ASCII FORMAT .... loaded.');
elseif strcmp(ext,'.edf')
  [hdr, NEdata] = edfRead(filepath);
   disp('Data in EDF FORMAT .... loaded.');
  NEdata=NEdata';
  % last column is time in ms:
  NEdata=[NEdata (1:length(NEdata))'*0 (1:length(NEdata))'/fs*1000]; 
else
  error('Unrecognized file extension')
end
 
%% Check if data needs transposing. We want a time sample per line/row: 
if (size(NEdata,1) < size(NEdata,2))
    NEdata=NEdata';
    warning('Data needs transposing ... done');   
end



%% check for number of EEG channels: 8 or 20? Or 32?
% if there are more than 20, it means that we have a 20 Ch device
if size(NEdata,2) > 32
  NumberOfChannels=32;
  disp('Enobio 32 device detected.');
  if ~exist('LabelsElectrodes','var') || isempty(LabelsElectrodes) 
    LabelsElectrodes={'Ch1','Ch2','Ch3','Ch4','Ch5','Ch6','Ch7','Ch8',...
        'Ch9','Ch10','Ch11','Ch12','Ch13','Ch14','Ch15','Ch16',...
        'Ch17','Ch18','Ch19','Ch20','Ch21','Ch22','Ch23','Ch24', ...
        'Ch25','Ch26','Ch27','Ch28','Ch29','Ch30','Ch31','Ch32'};
  end
elseif size(NEdata,2) > 20
  NumberOfChannels=20;
  disp('Enobio 20 device detected.');
  if ~exist('LabelsElectrodes','var') || isempty(LabelsElectrodes) 
    LabelsElectrodes={'P7','P4','Cz','Pz','P3','P8','O1','O2','T8','F8',...
        'C4','F4','Fp2','Fz','C3','F3','Fp1','T7','F7','EXT'};
  end
else
  if ~exist('LabelsElectrodes','var') || isempty(LabelsElectrodes) 
  LabelsElectrodes={'Ch1','Ch2','Ch3','Ch4','Ch5','Ch6','Ch7','Ch8'};
  end
  NumberOfChannels=8;
  disp('Enobio 8 or StarStim device detected.');
end



%% is the time interval specified?
 if exist('T','var')
     NEdata=NEdata((T(1)*fs+1):(T(2)*fs),:);
     disp(['Time boundaries set to >' num2str(T(1)) ' and <' num2str(T(2)) ' s.']); 
 else
     disp('Time boundaries: use all the file.')
 end
 
   
%% change units to uV (NE data is in nV)
NEdata=NEdata/1e3; % data is now in uV
timechannel=size(NEdata,2); % time is in ms

%% Configure time axis  
% Set time axis in seconds from last data column: 
time=(NEdata(:,timechannel)-NEdata(1,timechannel)); 



 

%% Now extract date/time of first point.
% from unixtime2mat.m ... to go from Unix to Matlab time
unix_epoch = datenum(1970,1,1,0,0,0);
matlab_time = NEdata(1,timechannel) ./86400 + unix_epoch;
StartDate=[datestr(matlab_time) ' UTC'];
disp(['File started (UTC): ' StartDate]);
% provide a handy time label for plots below:
TimeLabel= ['Time     (seconds from ' StartDate ')'];


%% Find out if we have AddSensor or Accelerometer
% Enobio8/20
%        10 Columns: no AddSensor, no accelerometer
%        11 Comumns: no accelerometer, but there is AddSensor
%        13 Columns: there is accelerometer, but no Addsensor
%        14 Comumns: there is accelerometer, AddSensor.
% Enobio 20 will have >= 22 and >=24 columns if accelerometer data is present.
%
%        22 Columns: no AddSensor, no accelerometer
%        23 Comumns: no accelerometer, but there is AddSensor
%        25 Columns: there is accelerometer, no AddSensor 
%        26 Comumns: there is accelerometer, AddSensor.
%
%   Enobio 32 will have >=34 adn >=36 if acc data is present
%        34 Columns: no AddSensor, no accelerometer
%        35 Comumns: no accelerometer, but there is AddSensor
%        37 Columns: there is accelerometer, no AddSensor 
%        38 Comumns: there is accelerometer, AddSensor.
%
%% this would be the case for StarStim:

AccDataIsPresent= false;
if sum(size(NEdata,2)==[13 14 25 26 37 38])>0  
    
   AccDataIsPresent= true;
   disp('Accelerometer Data is present.');
   AccelerationData=NEdata(:,NumberOfChannels+1:NumberOfChannels+3); % m/s^2
   disp('Calibrating accelerometer ...');
   diary OFF
   [AccelerationData x exitflag]=CalibrateAccelerometer(AccelerationData);
   diary(ProcessLogFile)

    
   if exitflag >0
    disp('Accelerometer Data calibrated successfully');
    disp(['Accelerometer bias =[' num2str(x) '] m/s^2']);
   else
    disp('Accelerometer Data calibration failed!');
   end

   
  AccLimit=0.5; %m/s^2
  disp(['Acceleration threshold is set to ' num2str(AccLimit) ' m/s']);
  
  % Let us plat raw data first:
  ha1=figure(177); 
  
  subplot(2,1,1)
  plot(time,AccelerationData);
  title('Acceleration Data'); 
    ylim([-12 12]);

  title(['Acceleration - NE data filename: ' filename],'Interpreter','none');
  legend('X','Y','Z') ;
  ylabel('m/s^2');
  xlabel('Time (s)');
   

  % magnitude of acceleration:
  MagAccelerationData=sqrt(sum(AccelerationData.^2,2));
  % removing the mean (mean=9.8 approx)
  MagAccelerationData=MagAccelerationData-mean(MagAccelerationData); 
  BadAccSpots=abs(MagAccelerationData)>AccLimit; % m/s^2
  
  subplot(2,1,2)
  plot(time, MagAccelerationData);
  hold on; plot(time(BadAccSpots), MagAccelerationData(BadAccSpots),'r^');
  plot(time,time.*0+AccLimit,'g--');plot(time,time.*0-AccLimit,'g--');
  title('Max acceleration');
  ylabel('m/s^2');
  xlabel(TimeLabel); 
  ylim([-2 2]);
  
  saveas(ha1,[figdir '/L-AccelerometerData.png']);
  saveas(ha1,[figdir '/L-AccelerometerData.fig']);

 if sum(BadAccSpots)> 0
  disp(['Warning: there were accelerometer events: Accelaration >'  ... 
                                               num2str(AccLimit) ' m/s']);
 end
 
 % Now let us plot PSD of acceleration data
a=spectrum.welch;
a.SegmentLength=4*fs;
 
 ha2=figure(178); 
for cha=1:3
 subplot(3,1,cha)
 PSDacc   =  psd(a,AccelerationData(:,cha) ,'Fs',fs,'ConfLevel',0.95);
 xlim([1 20]);
 plot(PSDacc); legend off;
 xlim([1 15]); 
 if (cha==1)
     ylabel('X axis '); 
     title(['NE data filename: ' filename],'Interpreter','none');
 else if (cha==2)
         ylabel('Y axis ')

     else
        ylabel('Z axis Power (dB/Hz)')  
     end
 end
end
saveas(ha2,[figdir '/L-AccelerometerPSD.png']);
saveas(ha2,[figdir '/L-AccelerometerPSD.fig']);
 
  
  % Now the acceleration Spectrograms
SegmentLength = 4*fs; % i.e., a 1 second wind for Welching.
nfft=SegmentLength; % array resolution at 1 Hz
ha3=figure(179); 
for cha=1:3
     subplot(3,1,cha)
  [~,f,t,p] =spectrogram(AccelerationData(:,cha),SegmentLength,[],nfft,fs,'yaxis');
       pp=10*log10(abs(p));
 
 surf(t,f,pp,'EdgeColor','none');   
      axis xy; axis tight; colormap(jet); view(0,90);
      ylabel('Frequency (Hz)');
       if (cha==3)
       xlabel(TimeLabel);  
         colorbar('SouthOutside') 
       end
  ylim([1  15]);
  if (cha==2)
  title(['Spectrogram']); 
  end
  if (cha==1)
 title(['NE data filename: ' filename],'Interpreter','none');
  end
    % set  plot to have same colorbar scale
  caxis([-80 20]);
end
 saveas(ha3,[figdir '/L-AccelerometerSPCT.png']);
 saveas(ha3,[figdir '/L-AccelerometerSPCT.fig']);
  
end
% done with acceleration data 


 
 
%% Plot raw data, all channels for a high level quick look at data 'as is"
h=figure(211); hold off; 
title(['Raw data: ' filepath]);
% change data units to mV:
Y=NEdata/1e3; %
 subplot(2,1,1)
 plot(time, Y(:,1:NumberOfChannels))
 title(['NE data filename: ' filename],'Interpreter','none');
 ylabel('Raw data (mV) - no reference');
 legend(LabelsElectrodes);
 subplot(2,1,2)
 plot(time, detrend(1000*Y(:,1:NumberOfChannels)));
 xlabel(TimeLabel); ylabel('Detrended Raw data (uV) - no reference');
% and save figure:
saveas(h,[figdir '/0-DetrendedRawdata, ref=none.png']);
saveas(h,[figdir '/0-DetrendedRawdata, ref=none.fig']);
 
 
%% Optionally plot referenced raw data, all channels
referenceChannel=Y(:,1).*0; %the default.

if (ref ~= 0)
 h=figure(200); hold off; 
 title(['Referenced Raw data: ' filepath]);
 % reference the data: 
    if ref==999
        referenceChannel=mean(Y,2);
    else
        referenceChannel=Y(:,ref);
       % This is the data in mV!
    end
    
    Z=Y-repmat(referenceChannel,1, timechannel) ;
    
    
  subplot(2,1,1)
  plot(time, Z(:,1:NumberOfChannels))
 title(['NE data filename: ' filename],'Interpreter','none');
  ylabel('Raw data (mV) - referenced');
  legend(LabelsElectrodes);
  subplot(2,1,2)
  plot(time, detrend(1000*Z(:,1:NumberOfChannels)));
  xlabel(TimeLabel); ylabel('Detrended Raw data (uV) - referenced');
% save figure:
 saveas(h,[figdir '/0-Rawdata, ref='  num2str(ref)  '.png']);
 saveas(h,[figdir '/0-Rawdata, ref='  num2str(ref)  '.fig']);
end


%% Channel loop: to carry out basic analysis for each channel
channels=1:NumberOfChannels;
refis=1; % this means "use a reference" ... the default behavior

% check if want unreferenced data:   
if ref==0 % ...unless no reference desired
    refis=0;ref=1; % this means "do not reference"
else channels(channels==ref)=[]; % i.e., process all but the reference
end  
   
NEdataRefFilPack =[];

% loop over channels   
for Channel=channels
 disp(' ');
 disp(['Processing channel ' char(LabelsElectrodes(Channel))]);

 % plot demeaned but otherwise raw data for the chosen Channel
 h=figure(100); hold off; % grab figure handle, open figure
 set(h,'Position',[717 -12 718 310]);
 plot(time, NEdata(:,Channel)-mean(NEdata(:,Channel)),'k'); 
 hold on;
  
 title(['Demeaned data (' filename ') Channel=' ...
  char(LabelsElectrodes(Channel)) ', ref=' num2str(0) ], 'Interpreter','none');
  xlabel(TimeLabel);
  ylabel('V in uV (arbitrary reference)')
 
 hold off; 
  % Save figure as file in "figures" folder:
 saveas(h,[figdir '/A-Rawdata Channel='  ...
                       char(LabelsElectrodes(Channel)) ', ref=none.png']);
 
     
%% reference + detrend (substract ref channel and  detrend
 
 NEdataRef=detrend(NEdata(:,Channel)-refis*referenceChannel*1000); % in nV

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% STEP 2:  filter the data
 % if do not want to filter here: comment next and uncomment following one
 NEdataRefFil=NE_filter(NEdataRef,flow,fhigh,fs); 
 % NEdataRefFil=NEdataRef; 
 
 NEdataRefFilPack=[NEdataRefFilPack   NEdataRefFil];
  
 % compute standard deviation of this particular channel
 stdenobiodatarf=round(std(NEdataRefFil));
 
% alert user that the STD is too high for this stream
if stdenobiodatarf > 25 % uV   
 disp(['Bad Channel Warning: STD in filtered data of Channel ' ...
     char(LabelsElectrodes(Channel)) ' = ' ...
     num2str(round(stdenobiodatarf)) 'uV ===> STD > 25']) 
else
    disp(['STD in [selected band of Channel ' ...
     char(LabelsElectrodes(Channel)) ' = ' ...
     num2str(round(stdenobiodatarf)) 'uV']);
end
 
 
% plot histograms
 figure(47)
 [a,b]=hist(NEdataRefFil,200);
 h=plot(b,a);
 title(['Histogram Channel='   char(LabelsElectrodes(Channel)) ...
  ', ref=' num2str(ref*refis) ' - Filter: ' bandtitle ' Hz' ], 'Interpreter','none');
 xlabel([ 'Voltage (uV);  standard deviation = ' num2str(stdenobiodatarf)  ' uV' ]);  
 ylabel('Count');
 
 saveas(h,[figdir '/Z-histogram Channel='  ...
     char(LabelsElectrodes(Channel)) ', ref='  num2str(ref*refis)  '.png']);
   
   
 % plot temporal data (now after filtering)   
 h=figure(101); hold off;
 set(h,'Position',[8 -13 718 310]);
 plot(time, NEdataRefFil,'k'); hold on;
 title(['Detrended, filtered data (' filename '), ' ... 
     ' Channel='   char(LabelsElectrodes(Channel)) ...
     ', ref=' num2str(ref*refis) ], 'Interpreter','none');
 xlabel([TimeLabel ';  standard deviation = ' ...
     num2str(stdenobiodatarf)  ' uV - Filter: ' bandtitle ' Hz' ]);  
 ylabel('V in uV (arbitrary reference)');
 ylim([ -100  100]);

 % 
 xlim([0 10]); % display 5 sec of data
 saveas(h,[figdir '/B-Cleandata-first10s Channel='  ...
     char(LabelsElectrodes(Channel)) ', ref=' ...
                                             num2str(ref*refis)  '.png']);
 xlim([5 10]); % display 5 sec of data
 saveas(h,[figdir '/B-Cleandata-5s Channel='  ...
     char(LabelsElectrodes(Channel)) ', ref=' ...
                                             num2str(ref*refis)  '.png']);
 
 xlim([5 25]); % display 20 seconds of data
 saveas(h,[figdir '/B-Cleandata-20s Channel=' ...
     char(LabelsElectrodes(Channel)) ', ref=' ...
                                             num2str(ref*refis)  '.png']);
 xlim([5 65]); % display 60 seconds of data
 
 
 saveas(h,[figdir '/B-Cleandata-60s Channel='  ...
     char(LabelsElectrodes(Channel)) ', ref=' ...
       num2str(ref*refis)  '.png']);
       
   
 % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%  Medical version of figure (strong visualization filters)
  hm=figure(203);
  set(hm,'Position',[8 -13 718 310]);

  % define frequency bandpass
  flowh=2; %Hz - low cutoff frequency for filtering
  fhighh=15; %Hz - high cutoff frequency for filtering
  
 NEdataRefFil2=NE_filter(NEdataRef,flowh,fhighh,fs);    
     
 % added the negative sign.... 
 
 invert=1;
 if mod(Channel,2)==0 
 plot(time, invert.*NEdataRefFil2+Channel*100,'b');
 else 
 plot(time, invert.*NEdataRefFil2+Channel*100,'r');
 end
   
  hold on;
   
   
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% STEP 3: Compute power density spectra (PSD)
  a=spectrum.welch; % this means use welch with hamming and 50% overalap
  a.SegmentLength=4*fs; % in time steps: this fixes resolution in spectrum  
  NE_PSD    =  psd(a,NEdataRefFil,'Fs',fs,'ConfLevel',0.95);
   
  deltaf=NE_PSD.Frequencies(3)- NE_PSD.Frequencies(2); % freq. step
  % find indices where we are in the chosen bandpass [flow fhigh]
  freqs=find(NE_PSD.Frequencies >flow & NE_PSD.Frequencies <fhigh);
  
  % Total power in the band [flow  fhigh] (integrate)
  IEnobioPSD=round(sum(NE_PSD.Data(freqs))*deltaf); % units are uV2
   
% Now we plot Spectra:
  h=figure(102) ; hold off; % figure handle
% make the figure size predictable and large
 
 plot(NE_PSD); 
 title(['PSD (' filename  ') Channel='  char(LabelsElectrodes(Channel)) ', ref=' ...
  num2str(ref*refis) ' -> Total Power (uV^2) = ' ...
  num2str(IEnobioPSD)], 'Interpreter','none');        
 xlim([flow-1 fhigh]);   
 ylim([-15 20]); % use this normally
 ylim([-30 0]); % use this for test bench analysis
 ylim([-40 80]); % use this for test bench analysis

% create png figure and put them in "figures" directory
 saveas(h,[figdir '/C-PSD Channel='   char(LabelsElectrodes(Channel))  ', ref=' ...
                                             num2str(ref*refis)  '.png']);                
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% STEP 4: Compute and plot spectrograms 
 window = fs; % i.e., a 1 second wind for Welching.
 nfft=window; % array resolution at 1 Hz
 
    [~,f,t,p] =spectrogram(NEdataRefFil,window,[],nfft,fs,'yaxis');
% 
if exist('h103') 
     close(h103);
 end
 h103=figure(103); hold off; % grab figure handle
 set(h103,'Position',[7 253 783 530]);
 hs1=subplot(2,1,1); % subplot handle
 pp=10*log10(abs(p));
 pp(pp<-20)=NaN;
 
 surf(t,f,pp,'EdgeColor','none');   
      axis xy; axis tight; colormap(jet); view(0,90);
      xlabel(TimeLabel);
      ylabel('Frequency (Hz)');
      colorbar('NorthOutside');
  ylim([flow-1 fhigh]);
  title(['Spectrogram (log10(abs(PSD): Channel='  ...
                                       char(LabelsElectrodes(Channel)) ...
              ', ref=' num2str(ref*refis) ' - Filter: ' bandtitle ' Hz' ]); 
  % set  plot to have same colorbar scale
  caxis([-15 20]);
  
  subplot(2,1,2)
  plot(time, NEdataRefFil,'k'); 
  xlabel([ TimeLabel ';  STD = '  num2str(stdenobiodatarf)  ' uv ' ]);   
  ylabel('V in uV (arbitrary reference)');
    ylim([ -100  100]);

   title(['Detrended, filtered data (' filename '), ' ... 
     ' Channel='   char(LabelsElectrodes(Channel)) ', ref=' ...
                 num2str(ref*refis) ],  'Interpreter','none');
  
  % save figure to file:
  saveas(h103,[figdir '/D-Spectrogram' ' Channel=' char(LabelsElectrodes(Channel))  ...
                           ', ref=' num2str(ref*refis) ' .png']);
end     

 %%
 % create data package with time in last column. This we return:
 NEdataRefFilPack=[NEdataRefFilPack   time];
 
figure(203);
  title(['Detrended, filtered 2-15Hz  data (' filename '), ' ... 
     ' ref=' num2str(ref*refis) ], 'Interpreter','none');
    %xlim([0 20]); % d 
    xlim([5 25]); % d 

   xlabel( TimeLabel);  
 ylabel('V in uV (50 uV per minor div)');

 if NumberOfChannels == 20
      set(gca,'YTick',100:100:2000)
      ylim([ 0  2100]);
 else
      set(gca,'YTick',100:100:800)
      ylim([ 0  900]);
 end
 
 set(gca,'YTickLabel',LabelsElectrodes)
 grid minor 
 grid 
     

  
 saveas(hm,[figdir '/M-Cleandata-20s Multi , ref=' ...
       num2str(ref*refis)  '.png']);
   
   disp(['Processing finished ' datestr(now)]);

   diary OFF
end % end of function







function xfilt=NE_filter(x,fL,fH,fs)
% WHAT:   NE_filter(x,fL,fH,fs) is a simple filter to bandpass EEG
%         must provide x (time series), fL (low f), fH (high f) 
%         and sampling rate (fs). Uses firpm function from Matlab
% INPUTS: time series x, high and low frequency cutoffs fL and fH, plus the
%         sampling frequency fs (in Hz)
%
% OUTPUTS: the filtered time series
%
% EXAMPLE: xfilt=NE_filter(x,8,12,250)

%   Author(s): G. Ruffini
%   Copyright 2011 Neuroelectrics Barcelona SL
%   $Revision: 0.1 $  $Date: 2011/11/26 $ 

%   References:
%     [1] Matlab help

rampspace=1; % in Hz this is the frequency internval allowed to go from 
             % pedestal to top or back down
             
N=2*fs; % sample points to remove on each side ... and filter length!

f= [0 fL-rampspace fL fH fH+rampspace fs/2]*2/fs;
a = [0 0 1 1 0  0]; % Band Pass filter 
% f and a  specify the filter in 4 points. The shape of the
% filter is of a truncated triangle...

b = firpm(N,f,a); 

xfilt=filter(b,1,x);
% remove beginning points to avoid artefacts.
xfilt(1:N)=0; % these are the critical ones.

end





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [hdr, record] = edfRead(fname, varargin)
% Read European Data Format file into MATLAB
%
% [hdr, record] = edfRead(fname)
%         Reads data from ALL RECORDS of file fname ('*.edf'). Header
%         information is returned in structure hdr, and the signals
%         (waveforms) are returned in structure record, with waveforms
%         associated with the records returned as fields titled 'data' of
%         structure record.
%
% [...] = edfRead(fname, 'assignToVariables', assignToVariables)
%         Triggers writing of individual output variables, as defined by
%         field 'labels', into the caller workspace.
%
% FORMAT SPEC: Source: http://www.edfplus.info/specs/edf.html SEE ALSO:
% http://www.dpmi.tu-graz.ac.at/~schloegl/matlab/eeg/edf_spec.htm
%
% The first 256 bytes of the header record specify the version number of
% this format, local patient and recording identification, time information
% about the recording, the number of data records and finally the number of
% signals (ns) in each data record. Then for each signal another 256 bytes
% follow in the header record, each specifying the type of signal (e.g.
% EEG, body temperature, etc.), amplitude calibration and the number of
% samples in each data record (from which the sampling frequency can be
% derived since the duration of a data record is also known). In this way,
% the format allows for different gains and sampling frequencies for each
% signal. The header record contains 256 + (ns * 256) bytes.
%
% Following the header record, each of the subsequent data records contains
% 'duration' seconds of 'ns' signals, with each signal being represented by
% the specified (in the header) number of samples. In order to reduce data
% size and adapt to commonly used software for acquisition, processing and
% graphical display of polygraphic signals, each sample value is
% represented as a 2-byte integer in 2's complement format. Figure 1 shows
% the detailed format of each data record.
%
% DATA SOURCE: Signals of various types (including the sample signal used
% below) are available from PHYSIONET: http://www.physionet.org/
%
%
% % EXAMPLE 1:
% % Read all waveforms/data associated with file 'ecgca998.edf':
%
% [header, recorddata] = edfRead('ecgca998.edf');
%
% % EXAMPLE 2:
% % Read records 3 and 5, associated with file 'ecgca998.edf':
%
% header = edfRead('ecgca998.edf','AssignToVariables',true);
% % Header file specifies data labels 'label_1'...'label_n'; these are
% % created as variables in the caller workspace.
%
% Coded 8/27/09 by Brett Shoelson, PhD
% brett.shoelson@mathworks.com
% Copyright 2009 - 2012 MathWorks, Inc.

% HEADER RECORD
% 8 ascii : version of this data format (0)
% 80 ascii : local patient identification
% 80 ascii : local recording identification
% 8 ascii : startdate of recording (dd.mm.yy)
% 8 ascii : starttime of recording (hh.mm.ss)
% 8 ascii : number of bytes in header record
% 44 ascii : reserved
% 8 ascii : number of data records (-1 if unknown)
% 8 ascii : duration of a data record, in seconds
% 4 ascii : number of signals (ns) in data record
% ns * 16 ascii : ns * label (e.g. EEG FpzCz or Body temp)
% ns * 80 ascii : ns * transducer type (e.g. AgAgCl electrode)
% ns * 8 ascii : ns * physical dimension (e.g. uV or degreeC)
% ns * 8 ascii : ns * physical minimum (e.g. -500 or 34)
% ns * 8 ascii : ns * physical maximum (e.g. 500 or 40)
% ns * 8 ascii : ns * digital minimum (e.g. -2048)
% ns * 8 ascii : ns * digital maximum (e.g. 2047)
% ns * 80 ascii : ns * prefiltering (e.g. HP:0.1Hz LP:75Hz)
% ns * 8 ascii : ns * nr of samples in each data record
% ns * 32 ascii : ns * reserved

% DATA RECORD
% nr of samples[1] * integer : first signal in the data record
% nr of samples[2] * integer : second signal
% ..
% ..
% nr of samples[ns] * integer : last signal

if nargin > 3
    error('EDFREAD: Too many input arguments.');
end

if ~nargin
    error('EDFREAD: Requires at least one input argument (filename to read).');
end

if nargin == 1
    assignToVariables = false;
end

[fid,msg] = fopen(fname,'r');
if fid == -1
    error(msg)
end

assignToVariables = false; %Default
for ii = 1:2:numel(varargin)
    switch lower(varargin{ii})
        case 'assigntovariables'
            assignToVariables = varargin{ii+1};
    end
end

% HEADER
hdr.ver        = str2double(char(fread(fid,8)'));
hdr.patientID  = fread(fid,80,'*char')';
hdr.recordID   = fread(fid,80,'*char')';
hdr.startdate  = fread(fid,8,'*char')';% (dd.mm.yy)
% hdr.startdate  = datestr(datenum(fread(fid,8,'*char')','dd.mm.yy'), 29); %'yyyy-mm-dd' (ISO 8601)
hdr.starttime  = fread(fid,8,'*char')';% (hh.mm.ss)
% hdr.starttime  = datestr(datenum(fread(fid,8,'*char')','hh.mm.ss'), 13); %'HH:MM:SS' (ISO 8601)
hdr.bytes      = str2double(fread(fid,8,'*char')');
reserved       = fread(fid,44);
hdr.records    = str2double(fread(fid,8,'*char')');
hdr.duration   = str2double(fread(fid,8,'*char')');
% Number of signals
hdr.ns    = str2double(fread(fid,4,'*char')');
for ii = 1:hdr.ns
    hdr.label{ii} = fread(fid,16,'*char')';
end
for ii = 1:hdr.ns
    hdr.transducer{ii} = fread(fid,80,'*char')';
end
% Physical dimension
for ii = 1:hdr.ns
    hdr.units{ii} = fread(fid,8,'*char')';
end
% Physical minimum
for ii = 1:hdr.ns
    hdr.physicalMin(ii) = str2double(fread(fid,8,'*char')');
end
% Physical maximum
for ii = 1:hdr.ns
    hdr.physicalMax(ii) = str2double(fread(fid,8,'*char')');
end
% Digital minimum
for ii = 1:hdr.ns
    hdr.digitalMin(ii) = str2double(fread(fid,8,'*char')');
end
% Digital maximum
for ii = 1:hdr.ns
    hdr.digitalMax(ii) = str2double(fread(fid,8,'*char')');
end
for ii = 1:hdr.ns
    hdr.prefilter{ii} = fread(fid,80,'*char')';
end
for ii = 1:hdr.ns
    hdr.samples(ii) = str2double(fread(fid,8,'*char')');
end
for ii = 1:hdr.ns
    reserved    = fread(fid,32,'*char')';
end
hdr.label = deblank(hdr.label);
hdr.units = deblank(hdr.units);


if nargout > 1 || assignToVariables
    % Scale data (linear scaling)
    scalefac = (hdr.physicalMax - hdr.physicalMin)./(hdr.digitalMax - hdr.digitalMin);
    dc = hdr.physicalMax - scalefac .* hdr.digitalMax;
    
    % RECORD DATA REQUESTED
    tmpdata = struct;
    for recnum = 1:hdr.records
        for ii = 1:hdr.ns
            % Use a cell array for DATA because number of samples may vary
            % from sample to sample
            tmpdata(recnum).data{ii} = fread(fid,hdr.samples(ii),'int16') * scalefac(ii) + dc(ii);
        end
    end
    record = zeros(hdr.ns, hdr.samples(1)*hdr.records);
    
    for ii = 1:numel(hdr.label)
        ctr = 1;
        for jj = 1:hdr.records
            try
                record(ii, ctr : ctr + hdr.samples - 1) = tmpdata(jj).data{ii};
            end
            ctr = ctr + hdr.samples;
        end
    end
    
    if assignToVariables
        for ii = 1:numel(hdr.label)
            try
                eval(['assignin(''caller'',''',hdr.label{ii},''',record(ii,:))'])
            end
        end
    end
end
fclose(fid);
 
end



function [CalAccData x exitflag]=CalibrateAccelerometer(AccData)


% banana=@(x) ...
%     sum(... % sum over time
%     sqrt(...
%     sum( (AccData -repmat([x(1) x(2) x(3)], length(AccData),1)).^2,  2) ...
%        ) -9.8 ....
%     )/length(AccData);

banana=@(x) ...
    sqrt(...
    sum( (AccData -repmat([x(1) x(2) x(3)], length(AccData),1)).^2,  2) ...
       ) -9.8 ;


[x,resnorm,residual,exitflag]  = lsqnonlin(banana,[0 0 -0.]);
 


CalAccData=AccData -repmat([x(1) x(2) x(3)], length(AccData),1);

end



